<?php
namespace AWeberWebFormPluginNamespace;

/**
    * AWeber Web Form Plugin object
    *
    * Main wordpress interface for integrating your AWeber Web Forms into
    * your blog.
    */
class AWeberWebformPlugin {
    var $adminOptionsName = 'AWeberWebformPluginAdminOptions';
    var $widgetOptionsName = 'AWeberWebformPluginWidgetOptions';
    var $oauthOptionsName = 'AWeberWebformOauth';
    var $messages = array();

    /**
        * Constructor
        */
    function __construct() {
        $aweber_settings_url = admin_url('admin.php?page=aweber.php');
        
        $this->messages['auth_required'] = 'AWeber Sign Up Form requires authentication. You will need you to update your <a href="' . $aweber_settings_url . '">settings</a> in order to continue to use AWeber Sign Up Form.';
        $this->messages['auth_error'] = 'AWeber Sign Up Form authentication failed.  Please verify the <a href="' . $aweber_settings_url . '">settings</a> to continue to use AWeber Sign Up Form.';
        $this->messages['auth_failed'] = '<div id="aweber_auth_failed" class="error">AWeber Sign Up Form authentication failed, <a href="' . $aweber_settings_url . '">please reconnect</a>.';
        $this->messages['signup_text_too_short'] = '<div id="aweber_signup_text_too_long" class="error">The subscriber label was too short. Please make sure it is at least 7 characters.</div>';
        $this->messages['no_list_selected'] = '<div id="aweber_no_list_selected" class="error">Your changes were not saved, as no list was selected.</div>';
        $this->messages['temp_error'] = '<div id="aweber_temp_error" class="error">Unable to connect to AWeber\'s API.  Please refresh the page, or <a href="' . admin_url('admin.php?page=aweber.php&reauth=true') . '">attempt to reauthorize.</a></div>';

        $this->ensure_defaults();
    }

    /**
        * Plugin initializer
        *
        * Main plugin initialization hook.
        * @return void
        */
    function init() {
    }

    function ensure_defaults() {
        $pluginAdminOptions = get_option($this->adminOptionsName);
        update_option('AWeberWebformPluginAdminOptions', array(
            'consumer_key'    => $pluginAdminOptions['consumer_key'],
            'consumer_secret' => $pluginAdminOptions['consumer_secret'],
            'access_key'      => $pluginAdminOptions['access_key'],
            'access_secret'   => $pluginAdminOptions['access_secret'],
        ));
        $options = get_option($this->widgetOptionsName);
        $keys = array(
            'list',
            'webform',
            'form_snippet',
            'selected_signup_form_list_id',
            'selected_split_test_form_list_id',
        );
        foreach ($keys as $key) {
            $options[$key] = isset($options[$key]) ? $options[$key] : '';
        }
        $keys = array(
            'create_subscriber_comment_checkbox' => 'ON',
            'create_subscriber_registration_checkbox' => 'ON',
            'aweber_comment_subscriber_text' => "Sign up to our newsletter!",
            'aweber_register_subscriber_text' => "Sign up to our newsletter!",
            'aweber_register_subscriber_listid' => null,
            'aweber_comment_subscriber_listid'  => null,
            'aweber_register_subscriber_tags'   => '',
            'aweber_comment_subscriber_tags'    => '',
            'aweber_add_analytics_checkbox' => 'OFF',
            'aweber_analytics_src'  => null,
            'create_sub_comment_ids' => array(),
        );
        foreach ($keys as $key => $value) {
            if (!isset($options[$key]) or ($options[$key] == null))
                $options[$key] = $value;
        }

        /* Delete OLD Keys: Code will be executed, only if the create_subscriber_signup_text key is set.  */
        if (isset($options['create_subscriber_signup_text'])):
            $options['aweber_comment_subscriber_text']  = $options['create_subscriber_signup_text'];
            $options['aweber_register_subscriber_text'] = $options['create_subscriber_signup_text'];

            $options['aweber_register_subscriber_listid'] = $options['list_id_create_subscriber'];
            $options['aweber_comment_subscriber_listid']  = $options['list_id_create_subscriber'];

            unset($options['create_subscriber_signup_text']);
            unset($options['list_id_create_subscriber']);

            delete_option('aweber_signup_text_value');
            update_option('aweber_webform_oauth_removed', 'FALSE');
        endif;
        update_option($this->widgetOptionsName, $options);
    }

    // Create the function to output the contents of our Dashboard Widget

    function aweber_dashboard_widget_function() {
        $this->aweber_wp_dashboard_cached_rss_widget('aweber_dashboard_widget', 'wp_dashboard_rss_output');
        $pluginAdminOptions = get_option($this->adminOptionsName);
        $options = get_option($this->widgetOptionsName);
        if ($pluginAdminOptions['access_key']) {
            extract($pluginAdminOptions);
            $error_code = $description = '';
            try {
                $aweber = $this->_get_aweber_api($consumer_key, $consumer_secret);
                $account = $aweber->getAccount($access_key, $access_secret);
            } catch (AWeberException $e) {
                $error_ = get_class($e);
                $error_code  = $e->status;
                $description = $e->getMessage();
                if (stripos($description, 'labs.aweber.com') !== false) {
                    $description = preg_replace('/http.*$/i', '', $description);  # strip labs.aweber.com documentation url from error message
                }
                $account = null;
            } catch (AWeberAPIException $e){
                $description = $e->getMessage();
                $error_code  = $e->status;
                if (stripos($description, 'labs.aweber.com') !== false) {
                    $description = preg_replace('/http.*$/i', '', $description);  # strip labs.aweber.com documentation url from error message
                }
                $account = null;
            } catch (AWeberOAuthDataMissing $e){
                $account = null;
            } catch (\Exception $exc) {
                $description = $exc->getMessage();
                $account = null;
            }
            if (!$account) {
                $this->displayCustomErrorMessages($error_code, $description);
            } else {
                /*  Subscribers after leaving a comment.  */
                try {
                    if (is_numeric($options['aweber_comment_subscriber_listid'])) {
                        $list = $account->loadFromUrl('/accounts/' . $account->id . '/lists/' . $options['aweber_comment_subscriber_listid']);
                    ?>
                    <ul>
                        <li>
                            <strong>List name: </strong><?php echo $list->name;?> <br>
                        </li>
                        <li>
                            <strong>Subscribed today to this list: </strong><?php echo $list->total_subscribers_subscribed_today;?> <br>
                        </li>
                        <li>
                            <strong>Subscribed yesterday to this list: </strong><?php echo $list->total_subscribers_subscribed_yesterday;?> <br>
                        </li>
                        <li>
                            <strong>Total subscribers on this list: </strong><?php echo $list->total_subscribed_subscribers;?> <br>
                        </li>
                    </ul>
                    <?php
                    }
                } catch (AWeberAPIException $exc) {
                    #List ID was not in this account
                    if ($exc->type === 'NotFoundError') {
                        $options = get_option($this->widgetOptionsName);
                        $options['aweber_comment_subscriber_listid'] = null;
                        update_option($this->widgetOptionsName, $options);
                    }
                }

                /*  Subscribers when register to the blog.  */
                try {
                    if (is_numeric($options['aweber_register_subscriber_listid'])) {
                        $list = $account->loadFromUrl('/accounts/' . $account->id . '/lists/' . $options['aweber_register_subscriber_listid']);
                    ?>
                    <ul>
                        <li>
                            <strong>List name: </strong><?php echo $list->name;?> <br>
                        </li>
                        <li>
                            <strong>Subscribed today to this list: </strong><?php echo $list->total_subscribers_subscribed_today;?> <br>
                        </li>
                        <li>
                            <strong>Subscribed yesterday to this list: </strong><?php echo $list->total_subscribers_subscribed_yesterday;?> <br>
                        </li>
                        <li>
                            <strong>Total subscribers on this list: </strong><?php echo $list->total_subscribed_subscribers;?> <br>
                        </li>
                    </ul>
                    <?php
                    }
                } catch (AWeberAPIException $exc) {
                    #List ID was not in this account
                    if ($exc->type === 'NotFoundError') {
                        $options = get_option($this->widgetOptionsName);
                        $options['aweber_register_subscriber_listid'] = null;
                        update_option($this->widgetOptionsName, $options);
                    }
                }
            }
        }
        else {
        }
        ?>
            </br>
            <a href="https://www.aweber.com/login.htm" target="_blank">Login to AWeber</a>
        <?php
    }

    /* Copied from WP code
    /  Modified to remove doing_AJAX global
    */
    function aweber_wp_dashboard_cached_rss_widget( $widget_id, $callback, $check_urls = array() ) {
        $loading = '<p class="widget-loading hide-if-no-js">' . __( 'Loading&#8230;' ) . '</p><p class="hide-if-js">' . __( 'This widget requires JavaScript.' ) . '</p>';

        if ( empty($check_urls) ) {
            $widgets = get_option( 'dashboard_widget_options' );
            if ( empty($widgets[$widget_id]['url']) ) {
                echo $loading;
                return false;
            }
            $check_urls = array( $widgets[$widget_id]['url'] );
        }

        $cache_key = 'dash_' . md5( $widget_id );
        if ( false !== ( $output = get_transient( $cache_key ) ) ) {
            echo $output;
            return true;
        }

        if ( $callback && is_callable( $callback ) ) {
            $args = array_slice( func_get_args(), 2 );
            array_unshift( $args, $widget_id );
            ob_start();
            call_user_func_array( $callback, $args );
            set_transient( $cache_key, ob_get_flush(), 43200); // Default lifetime in cache of 12 hours (same as the feeds)
        }

        return true;
    }

    // Create the function use in the action hook
    function aweber_add_dashboard_widgets() {
        $widget_options = get_option('dashboard_widget_options');
        if (!isset($widget_options['aweber_dashboard_widget'])) {
            $widget_options['aweber_dashboard_widget'] = array(
                'items' => 2,
                'show_summary' => 1,
                'show_author' => 0,
                'show_date' => 1,
            );
        }
        $widget_options['aweber_dashboard_widget']['link'] = apply_filters('aweber_dashboard_widget_link',  __('http://www.aweber.com/blog/'));
        $widget_options['aweber_dashboard_widget']['url'] = apply_filters('aweber_dashboard_widget_url',  __('http://www.aweber.com/blog/feed'));
        $widget_options['aweber_dashboard_widget']['title'] = apply_filters('aweber_dashboard_widget_title',  __('AWeber'));
        update_option('dashboard_widget_options', $widget_options);

        wp_add_dashboard_widget('aweber_dashboard_widget', $widget_options['aweber_dashboard_widget']['title'], array($this, 'aweber_dashboard_widget_function'), array($this, 'aweber_dashboard_widget_control'));
        // Globalize the metaboxes array, this holds all the widgets for wp-admin

        global $wp_meta_boxes;

        // Get the regular dashboard widgets array
        // (which has our new widget already but at the end)

        $normal_dashboard = $wp_meta_boxes['dashboard']['normal']['core'];

        // Backup and delete our new dashbaord widget from the end of the array

        $example_widget_backup = array('aweber_dashboard_widget' => $normal_dashboard['aweber_dashboard_widget']);
        unset($normal_dashboard['aweber_dashboard_widget']);

        // Merge the two arrays together so our widget is at the beginning

        $sorted_dashboard = array_merge($example_widget_backup, $normal_dashboard);

        // Save the sorted array back into the original metaboxes

        $wp_meta_boxes['dashboard']['normal']['core'] = $sorted_dashboard;
    }

    function aweber_dashboard_widget_control() {
        wp_dashboard_rss_control( 'aweber_dashboard_widget' );
    }

    function add_register_checkbox() {
        $this->add_checkbox('aweber_register_subscriber_text');
    }

    function add_comment_checkbox() {
        $this->add_checkbox('aweber_comment_subscriber_text');
    }

    function add_checkbox($key) {
        $options = get_option($this->widgetOptionsName);
        ?>
        <p>
        <input value="1" id="aweber_checkbox" type="checkbox" style="width:inherit;" name="aweber_signup_checkbox"/>
            <label for="aweber_checkbox">
            <?php echo $options[$key]; ?>
            </label>
        </p>
        </br>
        <?php
    }

    function deauthorize()
    {
        $admin_options = get_option($this->adminOptionsName);
        $admin_options = array(
            'consumer_key' => null,
            'consumer_secret' => null,
            'access_key' => null,
            'access_secret' => null,
        );
        update_option($this->adminOptionsName, $admin_options);
        $options = get_option($this->widgetOptionsName);
        $options['aweber_register_subscriber_listid'] = null;
        $options['aweber_comment_subscriber_listid'] = null;
        update_option($this->widgetOptionsName, $options);
        delete_option('aweber_webform_oauth_id');
        delete_option('aweber_webform_oauth_removed');
        delete_option('aweber_oauth_error');
    }

    function create_subscriber($email, $ip, $list_id, $name, $tags)
    {
        // Ignore the IP address if it looks malformed
        $ip = filter_var($ip, FILTER_VALIDATE_IP) or null;
        // Get the Tags for the subscriber. Return Empty array, if no tags. 
        $tags = empty($tags) ? array() : explode(',', $tags);
        $tags = array_map('trim', $tags);
        $tags = json_encode($tags);
        $admin_options = get_option($this->adminOptionsName);
        try {
            $aweber = $this->_get_aweber_api($admin_options['consumer_key'], $admin_options['consumer_secret']);
            $account = $aweber->getAccount($admin_options['access_key'], $admin_options['access_secret']);
            $subs = $account->loadFromUrl('/accounts/' . $account->id . '/lists/' . $list_id . '/subscribers');
            return $subs->create(array(
                                    'email' => $email,
                                    'tags'  => $tags,
                                    'ip_address' => $ip,
                                    'name' => $name,
                                    'ad_tracking' => 'Wordpress',
                                ));
        } catch (AWeberAPIException $exc){
            #List ID was not in this account
            if ($exc->type === 'NotFoundError') {
                $options = get_option($this->widgetOptionsName);
                $options['list_id_create_subscriber'] = null;
                if ($list_id == $options['aweber_register_subscriber_listid']) {
                    $options['aweber_register_subscriber_listid'] = null;
                }
                if ($list_id == $options['aweber_comment_subscriber_listid']) {
                    $options['aweber_comment_subscriber_listid'] = null;
                }
                update_option($this->widgetOptionsName, $options);
            }
            #Authorization is invalid
            if ($exc->type === 'UnauthorizedError')
                $this->deauthorize();
            
            // Logging the error in the Log file.
            error_log("Create Subscriber Error Occurred. Error Email: " . $email . " Type: " . $exc->type . " Message: " . $exc->message);
        } catch (\Exception $exc) {
            // Fail Silently and log the error in the Log file.
            $message = $exc->getMessage();
            $trace = var_export($exc->getTraceAsString(), true);

            error_log("Create Subscriber Error Occurred. Message: " . $message . " Trace: " . $trace);
        }
    }

    function comment_approved($comment)
    {
        $options = get_option($this->widgetOptionsName);
        $send_coi = $this->find_comment_id($comment->comment_ID);
        if ($send_coi) {
            $this->create_from_comment($comment);
        }
    }

    function find_comment_id($comment_id)
    {
        $options = get_option($this->widgetOptionsName);
        $index = array_search($comment_id, $options['create_sub_comment_ids']);
        if ($index !== false) {
            unset($options['create_sub_comment_ids'][$index]);
            #re-index the array
            $options['create_sub_comment_ids'] = array_values($options['create_sub_comment_ids']);
            update_option($this->widgetOptionsName, $options);
            return true;
        }
        else {
            return false;
        }
    }

    function comment_deleted($comment_id)
    {
        $this->find_comment_id($comment_id);
    }

    function create_from_comment($comment) {
        $options = get_option($this->widgetOptionsName);

        $email = $comment->comment_author_email;
        $name = $comment->comment_author;

        $sub = $this->create_subscriber($email, null, 
            $options['aweber_comment_subscriber_listid'], $name, 
            $options['aweber_comment_subscriber_tags']);
    }

    function grab_email_from_comment($comment_id, $comment = null)
    {
        if ($_POST['aweber_signup_checkbox'] != 1)
            return;

        $comment_id = (int) $comment_id;
        $comment = get_comment($comment_id);
        if(!$comment)
            return;

        $options = get_option($this->widgetOptionsName);

        if ($comment->comment_approved == 1)
            $this->create_from_comment($comment);
        else {
            if(count($options['create_sub_comment_ids']) >= 10000) {
                array_shift($options['create_sub_comment_ids']);
            }
            array_push($options['create_sub_comment_ids'], $comment->comment_ID);
            update_option($this->widgetOptionsName, $options);
        }
    }

    function get_request_ip()
    {
        if(array_key_exists('HTTP_X_REAL_IP', $_SERVER)) {
            return $_SERVER['HTTP_X_REAL_IP'];
        }
        if(array_key_exists('HTTP_X_FORWARDED_FOR', $_SERVER)) {
            return trim(explode(',', $_SERVER['HTTP_X_FORWARDED_FOR'], 2)[0]);
        }
        if(array_key_exists('REMOTE_ADDR', $_SERVER)) {
            return $_SERVER['REMOTE_ADDR'];
        }
        return null;
    }

    function grab_email_from_registration()
    {
        if ($_POST['aweber_signup_checkbox'] != 1)
            return;
        if(isset($_POST['user_email'])) {
            $email = $_POST['user_email'];
            $username = $_POST['user_login'];
            $ip = $this->get_request_ip();
            $options = get_option($this->widgetOptionsName); 
            $sub = $this->create_subscriber($email, $ip, 
                $options['aweber_register_subscriber_listid'], $username, 
                $options['aweber_register_subscriber_tags']);
        }
    }

    /**
        * Add content to the header tag.
        *
        * Hook for adding additional tags to the document's HEAD tag.
        * @return void
        */
    function addHeaderCode() {
        if (function_exists('wp_enqueue_script')) {
            // Admin page scripts
            if (is_admin()) {
                wp_enqueue_script('jquery');
            }
        }
    }

    /**
        * Get admin panel options.
        *
        * Retrieve admin panel settings variables as stored within wordpress.
        * @return array
        */
    function getAdminOptions() {
        $pluginAdminOptions = array(
            'consumer_key'    => null,
            'consumer_secret' => null,
            'access_key'      => null,
            'access_secret'   => null,
        );
        $options = get_option($this->adminOptionsName);
        if (!empty($options)) {
            foreach ($options as $key => $option) {
                $pluginAdminOptions[$key] = $option;
            }
        }
        update_option($this->adminOptionsName, $pluginAdminOptions);
        return $pluginAdminOptions;
    }

    /**
        * Print admin panel settings page.
        *
        * Echo the HTML for the admin panel settings page.
        * @return void
        */
    function printAdminPage() {
        $options = get_option($this->adminOptionsName);
        include(dirname(__FILE__) . '/aweber_forms_import_admin.php');
    }

    function printSignupInfo () {
        $options = get_option($this->adminOptionsName);
        require_once(dirname(__FILE__) . '/aweber_track_signup_form.php');
    }

    function printSystemInfo () {
        $options = get_option($this->adminOptionsName);
        require_once(dirname(__FILE__) . '/aweber_system_info.php');
    }

    function showConnectionPage() {
        require_once(dirname(__FILE__) . '/aweber_connection.php');
    }

    function attachAWeberheader() {
        require_once(dirname(__FILE__) . '/aweber_header_section.php');
    }

    function add_alert_message_html($message_type = '', $message = '', $extra_classes = '') {
        $css_class_names = $svg = '';
        if ($message_type == 'positive') {
            $css_class_names .= ' aweber-alert-positive';
            $svg = '<svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 24 24" width="24px" height="24px" xmlns:xlink="http://www.w3.org/1999/xlink" role="img" data-src="assets/toolkit/images/svg/check-circle.svg" class="injected-svg icon"><path d="M12 0C5.4 0 0 5.4 0 12s5.4 12 12 12 12-5.4 12-12S18.6 0 12 0zm6.8 8.9L11 16.7c-.2.2-.6.4-.9.4s-.7-.1-1-.4L5 12.6c-.5-.5-.5-1.3 0-1.8s1.3-.5 1.8 0L10 14l7-6.9c.5-.5 1.3-.5 1.8 0s.5 1.3 0 1.8z" role="presentation"></path></svg>';
        } else if($message_type == 'negative') {
            $css_class_names .= ' aweber-alert-negative';
            $svg = '<svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 24 24" width="24px" height="24px" xmlns:xlink="http://www.w3.org/1999/xlink" role="img" data-src="assets/toolkit/images/svg/alert.svg" class="injected-svg icon"><path d="M22.3 11.3l-9.6-9.6c-.4-.4-1-.4-1.4 0l-9.6 9.6c-.4.4-.4 1 0 1.4l9.6 9.6c.4.4 1 .4 1.4 0l9.6-9.6c.4-.4.4-1 0-1.4zM10.8 6.9c0-.7.6-1.2 1.2-1.2s1.2.6 1.2 1.2v6.4c0 .7-.6 1.2-1.2 1.2s-1.2-.6-1.2-1.2V6.9zM12 18.5c-.7 0-1.3-.6-1.3-1.3s.6-1.3 1.3-1.3 1.3.6 1.3 1.3-.6 1.3-1.3 1.3z" role="presentation"></path></svg>';
        }
        $css_class_names .= ' ' . $extra_classes;
        echo '<div class="aweber-alert ' . $css_class_names . '">' . $svg . '<p>'.  $message . '</p></div>';
    }

    /**
        * Auto load the AWeber Analytics script.
        *
        * Adds the analytics plugin in head tag
        * @return void
        */
    function loadAWeberPluginScripts() {
        $options = get_option($this->widgetOptionsName);

        //Add Analytics plugins only if: 1. AWeber connection exists 2. Auto add Analytics javascript is enabled 3. AWeber Analytics src not null  
        if (get_option('aweber_webform_oauth_removed')
            && isset($options['aweber_add_analytics_checkbox']) 
            && $options['aweber_add_analytics_checkbox'] == "ON" 
            && !empty($options['aweber_analytics_src'])) {

            // Parameters: $handle, $src, $deps, $ver, $in_footer
            wp_enqueue_script( 'script', $options['aweber_analytics_src'], false, null, false);
        }
    }

    /**
        * Get widget options.
        *
        * Retrieve widget control settings variables as stored within wordpress.
        * @return array
        */
    function getWidgetOptions() {
        $pluginWidgetOptions = array(
            'list'         => null,
            'webform'      => null,
            'form_snippet' => null,
            'list_id_create_subscriber' => null,
            'create_subscriber_comment_checkbox' => 'ON',
            'create_subscriber_registration_checkbox' => 'ON',
            'aweber_comment_subscriber_text' => "Sign up to our newsletter!",
            'aweber_register_subscriber_text' => "Sign up to our newsletter!",
            'aweber_register_subscriber_listid' => null,
            'aweber_comment_subscriber_listid'  => null,
            'aweber_add_analytics_checkbox' => 'OFF',
            'aweber_analytics_src'  => null,
            'create_sub_comment_ids' => array(),
        );
        $options = get_option($this->widgetOptionsName);
        if (!empty($options)) {
            foreach ($options as $key => $option) {
                $pluginWidgetOptions[$key] = $option;
            }
        }
        update_option($this->widgetOptionsName, $pluginWidgetOptions);
        return $pluginWidgetOptions;
    }

    /**
        * Fetch AWeber Shortcodes
        *
        * Retrives the AWeber shortcodes from the result passed and pushes into the Shortcode array.
        * @return Void
        */
    private function fetchShortCodesFromPOSTContent($results, &$shortcodes) {
        foreach($results as $result) {
            preg_match_all( '/\[aweber(.*?)\]/', $result->post_content, $matches);
            foreach ($matches[1] as $list) {
                $attributes = shortcode_parse_atts($list);

                if (!empty($attributes['formid'])) {
                    $location = array('type' => ucfirst($result->post_type), 'title' => $result->post_title, 'link' => get_permalink($result->ID));

                    if (!array_key_exists($attributes['formid'], $shortcodes)) {
                        $shortcodes[$attributes['formid']] = array('shortcode' => $list, 'areas' => array());
                    }
                    array_push($shortcodes[$attributes['formid']]['areas'], $location);
                }
            }
        }
    }

    /**
        * Fetch AWeber Shortcodes
        *
        * Retrives the AWeber shortcodes from the result passed and pushes into the Shortcode array.
        * @return Void
        */
    private function fetchShortCodesFromWidgetContent($results, &$shortcodes) {
        foreach($results as $result) {
            $widget_options = get_option($result->option_name);

            foreach ($widget_options as $key => $option) {
                $matches = preg_grep('/\[aweber(.*?)\]/', $option);
                foreach ($matches as $k => $list) {
                    $attributes = shortcode_parse_atts($list);
                    if (!empty($attributes['formid'])) {
                        $widget_name  = explode('_', $result->option_name)[1] . '-' . $key;
                        $location = array('type' => 'Shortcode Widget', 'title' => $this->getWidgetSidebarName($widget_name), 'link' => FALSE);

                        if (!array_key_exists($attributes['formid'], $shortcodes)) {
                            $shortcodes[$attributes['formid']] = array('shortcode' => $list, 'areas' => array());
                        }

                        if (array_search($location['title'], array_column($shortcodes[$attributes['formid']]['areas'], 'title')) === false) {
                            array_push($shortcodes[$attributes['formid']]['areas'], $location);
                        }
                    }
                }
            }
        }
    }

    /**
        * Fetch Shortcodes from sites
        *
        * Retrives the AWeber shortcodes from the wp_posts and wp_options table.
        * @return array
        */
    public function getShortCodeFromSite() {
        $shortcodes = array();

        global $wpdb;
        $args = array(
            'post_status'   => 'publish',
            'post_type'     => array('page', 'post'),
            'numberposts'   => 100,
            's'             => '[aweber ',
            'offset'        => 0
        );

        do {
            $result = get_posts($args);
            $this->fetchShortCodesFromPOSTContent($result, $shortcodes);
            $args['offset'] += count($result);
        } while (count($result) >= $args['numberposts']);

        $query = "SELECT option_name FROM " . $wpdb->prefix . "options 
            WHERE option_value LIKE '%[aweber %' AND option_name LIKE 'widget_%'";

        $this->fetchShortCodesFromWidgetContent($wpdb->get_results($query), $shortcodes);
        return $shortcodes;
    }

    function getSignupWebformsList() {
        $list_id = $_GET['list_id'];
        $type    = $_GET['type'];

        if (empty($list_id)) {
            echo json_encode(array('status' => 'error', 'message' => 'list id not found in the request.'));
            $this->_end_response();
        }

        $pluginAdminOptions = get_option($this->adminOptionsName);
        $options = get_option($this->widgetOptionsName);
    
        if ($pluginAdminOptions['access_key']):
            $error_code = $description = '';

            try {
                $aweber = $this->_get_aweber_api($pluginAdminOptions['consumer_key'], $pluginAdminOptions['consumer_secret']);
                $account = $aweber->getAccount($pluginAdminOptions['access_key'], $pluginAdminOptions['access_secret']);

                $sign_up_forms = $account->getWebForms();
            } catch (AWeberException $e) {
                $error_ = get_class($e);
                $error_code  = $e->status;
                $description = $e->getMessage();
                if (stripos($description, 'labs.aweber.com') !== false) {
                    $description = preg_replace('/http.*$/i', '', $description);  # strip labs.aweber.com documentation url from error message
                }
                $account = null;
            } catch (AWeberAPIException $e){
                $description = $e->getMessage();
                $error_code  = $e->status;
                if (stripos($description, 'labs.aweber.com') !== false) {
                    $description = preg_replace('/http.*$/i', '', $description);  # strip labs.aweber.com documentation url from error message
                }
                $account = null;
            } catch (AWeberOAuthDataMissing $e){
                $account = null;
            } catch (\Exception $exc) {
                $description = $exc->getMessage();
                $account = null;
            }

            if (!$account) {
                $message = $this->displayCustomErrorMessages($error_code, $description, 1);
                echo json_encode(array('status' => 'error', 'redirect' => admin_url('admin.php?page=aweber.php'),
                    'error_code' => $error_code, 'message' => $message));
                $this->_end_response();
            }

            $shortcodes = $this->getShortCodeFromSite();
            $getWidgetSidebarName = $this->getWidgetSidebarName($this->widgetOptionsName);

            $list_web_forms = array();

            $active_webform_id = explode('/', $options['webform'])[6];
            if ($type == 'sigup-form') {
                foreach ($sign_up_forms as $webform) {
                    if (explode('/', $webform->url)[4] == $list_id) {
                        $tag_lists = $webform->tags;
                        if(!empty($tag_lists)){
                            $tag_lists = implode(", ", iterator_to_array($tag_lists));
                        }
                        $con_perc = number_format((float)$webform->conversion_percentage, 2, '.', '');

                        $current_webform_id = explode('/', $webform->url)[6];
                        $widget_locations = array();
                        if ($active_webform_id == $current_webform_id && $getWidgetSidebarName != '-') {
                            array_push($widget_locations, array(
                                'type'  => 'Widget',
                                'title' => $getWidgetSidebarName,
                                'link'  => FALSE
                            ));
                        }
                        if (array_key_exists($current_webform_id, $shortcodes)) {
                            if (!empty($widget_locations)) {
                                array_push($shortcodes[$current_webform_id]['areas'], $widget_locations[0]);
                            }
                            $widget_locations = $shortcodes[$current_webform_id]['areas'];
                        }

                        array_push($list_web_forms, array(
                            'name'  => $webform->name,
                            'tags'  => $tag_lists,
                            'type'  => $webform->type,
                            'preview_form'  => $webform->html_source_link,
                            'displays'      => $webform->total_displays,
                            'submissions'   => $webform->total_submissions,
                            'conversion_rate' => $con_perc.'%',
                            'shortcode' => '[aweber listid="'.$list_id.'" formid="'.$current_webform_id.'" formtype="webform"]',
                            'location'  => json_encode($widget_locations)
                        ));
                    }
                }
                $options['selected_signup_form_list_id'] = $list_id;
                update_option($this->widgetOptionsName, $options);
            } elseif ($type == 'split-test') {
                foreach ($account->getWebFormSplitTests() as $webform) {
                    if (explode('/', $webform->url)[4] == $list_id) {
                        $components = $account->loadFromUrl($webform->data['components_collection_link']);
                        $test_webforms = array();

                        $current_webform_id = explode('/', $webform->url)[6];
                        $widget_locations = array();
                        if ($active_webform_id == $current_webform_id && $getWidgetSidebarName != '-') {
                            array_push($widget_locations, array(
                                'type'  => 'Widget',
                                'title' => $getWidgetSidebarName,
                                'link'  => FALSE
                            ));
                        }
                        if (array_key_exists($current_webform_id, $shortcodes)) {
                            if (!empty($widget_locations)) {
                                array_push($shortcodes[$current_webform_id]['areas'], $widget_locations[0]);
                            }
                            $widget_locations = $shortcodes[$current_webform_id]['areas'];
                        }

                        foreach ($components as $component) {
                            $tag_lists = $component->tags;
                            if(!empty($tag_lists)){
                                $tag_lists = implode(", ", iterator_to_array($tag_lists));
                            }
                            $s_d = '0.00';
                            $total_displays = $component->total_displays;
                            $total_submissions = $component->total_submissions;
                            $unique_displays = $component->total_unique_displays;
                            if (!empty(($total_displays)) && !empty(($total_submissions))) {
                                $s_d = number_format(100 * ($total_submissions / $total_displays), 2, '.', '');
                            }                            

                            $s_ud = '0.00';
                            if (!empty(($total_displays)) && !empty(($unique_displays))) {
                                $s_ud = number_format(100 * ($total_submissions / $unique_displays), 2, '.', '');
                            }
                            array_push($test_webforms, array(
                                'sign_up_form'  => $component->name,
                                'tags'          => $tag_lists,
                                'probability'   => $component->weight.'%',
                                'displays'      => $component->total_displays,
                                'subscribers'   => $component->total_submissions, 
                                's_d'           => $s_d.'%',
                                'unique_displays' => $component->total_unique_displays,
                                's_ud'          => $s_ud.'%'
                            ));
                        }
                        array_push($list_web_forms, array(
                            'test_name' => $webform->name,
                            'size'      => $components->total_size,
                            'location' => json_encode($widget_locations),
                            'shortcode' => '[aweber listid="'.$list_id.'" formid="'.$current_webform_id.'" formtype="split_tests"]',
                            'webform_split_tests'  => $test_webforms
                        ));
                    }
                }
                $options['selected_split_test_form_list_id'] = $list_id;
                update_option($this->widgetOptionsName, $options);

                usort($list_web_forms, function($a, $b) {
                    return strcmp($a['test_name'], $b['test_name']);
                });
            }
            
            if (empty($list_web_forms)):
                $response = array('status' => 'error', 'message' => 'You do not have any Sign Up Forms for the selected list.');
            else:
                $response = array('status' => 'success', 'webforms' => $list_web_forms);
            endif;
        else:
            $response = array('status' => 'error', 'message' =>  'Your connection with AWeber got closed. Please reconnect.');
        endif;

        echo json_encode($response);
        $this->_end_response();
    }

    function getAWeberWebformShortCodes(){
        $pluginAdminOptions = get_option($this->adminOptionsName);
        $options = get_option($this->widgetOptionsName);

        $error_code = $description = '';
        if ($pluginAdminOptions['access_key']):
            try {
                $aweber = $this->_get_aweber_api($pluginAdminOptions['consumer_key'], $pluginAdminOptions['consumer_secret']);
                $account = $aweber->getAccount($pluginAdminOptions['access_key'], $pluginAdminOptions['access_secret']);

                $account_list = $account->lists;
                $webforms = $account->getWebForms();

            } catch (AWeberException $e) {
                $error_code  = $e->status;
                $description = $e->getMessage();
                if (stripos($description, 'labs.aweber.com') !== false) {
                    $description = preg_replace('/http.*$/i', '', $description);  # strip labs.aweber.com documentation url from error message
                }
                $account = null;
            } catch (AWeberAPIException $e){
                $description = $e->getMessage();
                $error_code  = $e->status;
                if (stripos($description, 'labs.aweber.com') !== false) {
                    $description = preg_replace('/http.*$/i', '', $description);  # strip labs.aweber.com documentation url from error message
                }
                $account = null;
            } catch (\Exception $exc) {
                $description = $exc->getMessage();
                $account = null;
            }

            if (!$account) {
                $message = 'Please reconnect your AWeber account.';
                if ($error_code != '401') {
                    $message = $this->displayCustomErrorMessages($error_code, $description, 1);
                }
                echo json_encode(array('status' => 'error', 'message' => $message));
                $this->_end_response();
            }

            $lists = array();
            foreach ($account_list as $list) {
                $lists[$list->id] = $list->name;
            }

            $list_of_shortcodes = array();
            foreach ($webforms as $webform) {
                $list_id = explode('/', $webform->url)[4];
                if (array_key_exists($list_id, $lists)) {
                    array_push($list_of_shortcodes, array(
                        'text'      => $webform->name,
                        'list_id'   => $list_id,
                        'list_name' => $lists[$list_id],
                        'value' => $list_id . '-' . explode('/', $webform->url)[6] . '-webform'
                    ));
                }
            }
            foreach ($account->getWebFormSplitTests() as $webform) {
                $list_id = explode('/', $webform->url)[4];
                if (array_key_exists($list_id, $lists)) {
                    array_push($list_of_shortcodes, array(
                        'text'      => $webform->name,
                        'list_id'   => $list_id,
                        'list_name' => $lists[$list_id],
                        'value' => $list_id . '-' . explode('/', $webform->url)[6] . '-split_tests'
                    ));
                }
            }
            
            if (empty($list_of_shortcodes)):
                $response = array('status' => 'error', 'message' => 'No short codes found for the selected list.');
            else:
                usort($list_of_shortcodes, function($a, $b){
                    return $b['list_id'] - $a['list_id'];
                });

                $response = array('status' => 'success', 'short_codes' => $list_of_shortcodes);
            endif;
        else:
            $response = array('status' => 'error', 'message' => 'Please reconnect your AWeber account.');
        endif;

        echo json_encode($response);
        $this->_end_response();
    }

    private function getWidgetSidebarName($widget_name) {
        $widget_sidebar_name = '-';
        foreach (wp_get_sidebars_widgets() as $key => $value) {
            if (!empty($value) && in_array(strtolower($widget_name), $value)) {
                global $wp_registered_sidebars;

                foreach ($wp_registered_sidebars as $sidebars) {
                    if (!empty($sidebars['id']) && $sidebars['id'] == $key) {
                        $widget_sidebar_name = $sidebars['name'];
                        break;
                    }
                }
            }
        }
        return $widget_sidebar_name;
    }

    /**
        * Print error message in the browser.
        *
        * Echo the HTML with the respective error code and error message.
        * @return void
        */
    function displayCustomErrorMessages($response_error_code = '', $response_error_message = '', $return_message = 0) {
        $kb_link = "https://help.aweber.com/hc/en-us/articles/204027976-How-Do-I-Use-AWeber-s-Webform-Widget-For-Wordpress-";
        $message = '';
        switch ($response_error_code) {
            case '400':
                // Suppressed the error from the users.
                break;

            case '401':
                // Trim the whitespaces at the end.
                $response_error_message = trim($response_error_message);
                // Trim the dot at the end. So that the dot will not be duplicated in the message.
                $response_error_message = trim($response_error_message, '.');

                $message = 'Authorization not granted. (401) ' . $response_error_message . '. Please <a href="' . admin_url('admin.php?page=aweber.php&reauth=true') . '">click here</a> to reauthorize.';
                // Update the error in the tale.
                update_option('aweber_oauth_error', $message);
                if (stripos($message, 'expired timestamp') !== false) {
                    $message = 'An unexpected error occured. 
                        The clock time on your server doesn\'t appear to be the cuurent time.';
                }
                break;
            
            case '403':
                $message = 'An unexpected error occured: (403) ' . $response_error_message;
                if (stripos($response_error_message, 'suspended') !== false || stripos($response_error_message, 'hold') !== false) {
                    $message = 'Your AWeber account is not active or temporarily suspended.';
                } else if (stripos($response_error_message, 'at this time') !== false) {
                    // Suppressed the error from the users.
                    $message = '';
                }
                break;

            default:
                $message = "An unexpected error occured. (".$response_error_code.") " . $response_error_message;
                if (empty($response_error_code) && stripos($response_error_message, 'Authorization code entered') !== false) {
                    $message = $response_error_message;
                } else if (stripos($response_error_message, 'Connection time') !== false || stripos($response_error_message, 'http_request_failed') !== false) {
                    $message = 'An unexpected error occured. Please check your internet connection and try again.';
                }
                break;
        }
        if ($return_message) {
            return $message;
        }

        $message = $message . '<br><br> Please refer to the <a target="_blank" href="' . $kb_link . '">knowledge base
                </a> for assistance or email us at <a href="mailto:help@aweber.com">help@aweber.com</a>';
        $this->add_alert_message_html('negative', $message, 'aweber-body-wrapper');
    }

    /**
        * Print widget in sidepanel.
        *
        * Echo the HTML for the widget handle.
        * @return void
        */
    function printWidget($args) {
        extract($args, EXTR_SKIP);

        echo '<!-- Generated by the AWeber WordPress plugin -->';

        if (isset($before_widget) && !empty($before_widget)) {
            echo $before_widget;
        }

        if (isset($before_title) && !empty($before_title)) {
            echo $before_title;
        }
        if (isset($title) && !empty($title)) {
            echo $title;
        }
        if (isset($after_title) && !empty($after_title)) {
            echo $after_title;
        }

        echo $this->getWebformSnippet();

        if (isset($after_widget) && !empty($after_widget)) {
            echo $after_widget;
        }
    }

    /**
        * Get a new AWeber API object
        *
        * Wrapper for AWeber API generation
        * @return AWeberAPI
        */
    function _get_aweber_api($consumer_key, $consumer_secret) {
        return new AWeberAPI($consumer_key, $consumer_secret);
    }

    function getFormSnippet($account, $options) {
        $this_form = $account->loadFromUrl($options['webform']);
        $attrs = $this_form->attrs();
        if (isset($attrs['components'])) {
            $components = $account->loadFromUrl($this_form->components_collection_link);
            $componentIds = array();
            $options['form_snippet'] = '';
            foreach ($components as $component) {
                $component_form = $account->loadFromUrl($component->web_form_link);
                $componentIds[] = $component_form->id;
                $options['form_snippet'] .= '<div class="AW-Form-'.$component_form->id.'" style="display: none;"></div>';
            };
            $options['form_snippet'] .= '
                <script type="text/javascript">(function(d,s,id) {
                    var js;
                    var fjs = d.getElementsByTagName(s)[0];
                    var cids = [' . implode(',', $componentIds) .'];
                    var mos = [];
                    cids.forEach(function(cid) {
                        mo = new MutationObserver(function(m) {
                            m[0].target.style = "";
                            mos.forEach(function(omo) { omo.disconnect(); });
                        });
                        mo.observe(d.getElementsByClassName("AW-Form-"+cid)[0], {childList: true});
                        mos.push(mo);
                    });
                    if (d.getElementById(id)) return; js = d.createElement(s);
                    js.id = id; js.src = "'.$this->_getWebformJsUrl($this_form).'";
                    fjs.parentNode.insertBefore(js, fjs);
                    }(document, "script", "aweber-wjs-'.(string)rand().'"));
                </script>';
        } else {
            $options['form_snippet'] = '<div class="AW-Form-' . $this_form->id . '"></div>
                <script type="text/javascript">(function(d,s,id) {
                    var js;
                    var fjs = d.getElementsByTagName(s)[0];
                    if (d.getElementById(id)) return; js = d.createElement(s);
                    js.id = id; js.src = "' . $this->_getWebformJsUrl($this_form) . '";
                    fjs.parentNode.insertBefore(js, fjs);
                    }(document, "script", "aweber-wjs-' . (string)rand() . '"));
                </script>';
        }
        return $options;
    }

    function updateWebFormSnippet($options) {
        if (!empty($options['webform'])) {
            $admin_options = get_option($this->adminOptionsName);
            $aweber = $this->_get_aweber_api($admin_options['consumer_key'], $admin_options['consumer_secret']);
            try {
                $account = $aweber->getAccount($admin_options['access_key'], $admin_options['access_secret']);
            } catch (AWeberException $e) {
                $account = null;
            }
            if ($account) {
                $options = $this->getFormSnippet($account, $options);
            }
        } else {
            $options['form_snippet'] = '';
        }
        update_option($this->widgetOptionsName, $options);
    }

    /**
        * Print widget settings control in admin panel.
        *
        * Store settings and echo HTML for the widget control.
        * @return void
        */
    function printWidgetControl() {
        if (isset($_POST[$this->widgetOptionsName])) {
            $options = get_option($this->widgetOptionsName);
            $widget_data = $_POST[$this->widgetOptionsName];
            if (isset($widget_data['submit']) && $widget_data['submit']) {
                $options['list'] = $widget_data['list'];
                $options['webform'] = $widget_data[$widget_data['list']]['webform'];
                
                $this->updateWebFormSnippet($options);
            }
        } else {
            ?>
            <div id="<?php echo $this->widgetOptionsName; ?>-content" class="<?php echo $this->widgetOptionsName; ?>-content"><img src="images/loading.gif" height="16" width="16" id="aweber-webform-loading" style="float: left; padding-right: 5px" /> Loading...</div>
            <script type="text/javascript" >

            jQuery(document).ready(function($) {
                if (typeof(<?php echo $this->widgetOptionsName; ?>) != 'undefined') { return; }
                <?php echo $this->widgetOptionsName; ?> = true;

                var data = {
                    action: 'get_widget_control'
                };

                // since 2.8 ajaxurl is always defined in the admin header and points to admin-ajax.php
                jQuery.post(ajaxurl, data, function(response) {
                    var primary_content = jQuery('.<?php echo $this->widgetOptionsName; ?>-content');
                    primary_content.each(function() { jQuery(this).html(response); });
                });
            });
            </script>
            <?php
        }
    }

    /**
        * Get Web Form javascript url
        *
        * Returns hosted javascript url of a given form.
        * @param AWeberEntry
        * @return string
        */
    function _getWebformJsUrl($webform) {
        $form_hash = $webform->id % 100;
        $form_hash = (($form_hash < 10) ? '0' : '') . $form_hash;
        $prefix = ($this->_isSplitTest($webform)) ? 'split_' : '';
        return 'https://forms.aweber.com/form/' . $form_hash . '/' . $prefix . $webform->id . '.js';
    }

    function reloadWidgetWebForm() {
        if (isset($_GET[$this->widgetOptionsName])) {
            $options = get_option($this->widgetOptionsName);

            $this->updateWebFormSnippet($options);
            $response = array('status' => 'success', 'message' => 'Cache reloaded successfully');
        } else {
            $response = array('status' => 'error', 'message' => 'You cannot access the API directly.');
        }
        echo json_encode($response);
        $this->_end_response();
    }

    function aweberShortcodeHandler($attr) {
        if (empty($attr['formid'])) {
            return "Form Id not found. Please copy paste the correct shortcode text.";
        }
        if (empty($attr['listid'])) {
            return "List Id not found. Please copy paste the correct shortcode text.";
        }
        if (empty($attr['formtype']) || ($attr['formtype'] != 'webform' && $attr['formtype'] != 'split_tests')) {
            return "Form Type not found. Please copy paste the correct shortcode text.";
        }

        $option_name = $attr['listid'] . '-' . $attr['formid'];
        $options = get_option($option_name);
        if (empty($options['form_snippet'])) {
            $admin_options = get_option($this->adminOptionsName);
            try {
                $aweber = $this->_get_aweber_api($admin_options['consumer_key'], $admin_options['consumer_secret']);
                $account = $aweber->getAccount($admin_options['access_key'], $admin_options['access_secret']);
            } catch (AWeberException $e) {
                $account = null;
            }
            if (!$account) {
                return $this->messages['auth_error'];
            }

            $webform = '/accounts/' . $account->id . '/lists/' . $attr['listid'] . '/web_forms/' . $attr['formid'];
            if ($attr['formtype'] == 'split_tests') {
                $webform = '/accounts/' . $account->id . '/lists/' . $attr['listid'] . '/web_form_split_tests/' . $attr['formid'];
            }
            
            $options = array(
                'list'      => $attr['listid'],
                'webform'   => $webform,
                'form_snippet'  => ''
            );

            try{
                $options = $this->getFormSnippet($account, $options);
            } catch (AWeberAPIException $e) {
                return "AWeber Exception Occured: " . $e->type;
            }
            update_option($option_name, $options);
        }
        return $options['form_snippet'];
    }

    /**
        * Is a split test?
        *
        * Returns whether form object is a splittest.
        * @param AWeberEntry
        * @return bool
        */
    function _isSplitTest($webform) {
        return $webform->type == 'web_form_split_test';
    }

    function _end_response() {
        die();
    }

    /**
        * Response to be given to print action via AJAX.
        *
        * Echo HTML for widget control form asynchronously.
        * @return void
        */
    function printWidgetControlAjax() {
        $options = get_option($this->widgetOptionsName);
        $admin_options = get_option($this->adminOptionsName);

        // Render form
        $list = $options['list'];
        $webform = $options['webform'];

        try {
            $aweber = $this->_get_aweber_api($admin_options['consumer_key'], $admin_options['consumer_secret']);
            $account = $aweber->getAccount($admin_options['access_key'], $admin_options['access_secret']);
        } catch (AWeberException $e) {
            $account = null;
        }
        if (!$account) {
            echo $this->messages['auth_error'];
            return $this->_end_response();
        }

        $list_web_forms = array();
        foreach ($account->getWebForms() as $this_webform) {
            $link_parts = explode('/', $this_webform->url);
            $list_id = $link_parts[4];
            if (!array_key_exists($list_id, $list_web_forms)) {
                $list_web_forms[$list_id] = array(
                    'web_forms' => array(),
                    'split_tests' => array()
                );
            }
            $list_web_forms[$list_id]['web_forms'][] = $this_webform;
        }
        foreach ($account->getWebFormSplitTests() as $this_webform) {
            $link_parts = explode('/', $this_webform->url);
            $list_id = $link_parts[4];
            if (!array_key_exists($list_id, $list_web_forms)) {
                $list_web_forms[$list_id] = array(
                    'web_forms' => array(),
                    'split_tests' => array()
                );
            }
            $list_web_forms[$list_id]['split_tests'][] = $this_webform;
        }
        $lists = $account->lists;
        foreach ($lists as $this_list) {
            if (array_key_exists($this_list->id, $list_web_forms)) {
                $list_web_forms[$this_list->id]['list'] = $this_list;
            }
        }

        // The HTML form will go here
?>
<?php if (!empty($list_web_forms)): ?>
<select class="widefat <?php echo $this->widgetOptionsName; ?>-list" name="<?php echo $this->widgetOptionsName; ?>[list]" id="<?php echo $this->widgetOptionsName; ?>-list" style="margin-top: 13px; margin-bottom: 13px;">
    <option value="">Step 1: Select A List</option>
    <?php foreach ($list_web_forms as $this_list_data): ?>
    <?php $this_list = $this_list_data['list']; ?>
    <option value="<?php echo $this_list->id; ?>"<?php echo ($this_list->id == $list) ? ' selected="selected"' : ""; ?>><?php echo $this_list->name; ?></option>
    <?php endforeach; ?>
</select>

<?php foreach ($list_web_forms as $this_list_id => $forms): ?>
<select class="widefat <?php echo $this->widgetOptionsName; ?>-form-select <?php echo $this->widgetOptionsName; ?>-<?php echo $this_list_id; ?>-webform" name="<?php echo $this->widgetOptionsName; ?>[<?php echo $this_list_id; ?>][webform]" id="<?php echo $this->widgetOptionsName; ?>-<?php echo $this_list_id; ?>-webform" style="margin-bottom: 13px;">
    <option value="">Step 2: Select A Sign Up Form</option>
    <?php foreach ($forms['web_forms'] as $this_form): ?>
    <option value="<?php echo $this_form->url; ?>"<?php echo ($this_form->url == $webform) ? ' selected="selected"' : ''; ?>><?php echo $this_form->name; ?></option>
    <?php endforeach; ?>
    <?php foreach ($forms['split_tests'] as $this_form): ?>
    <option value="<?php echo $this_form->url; ?>"<?php echo ($this_form->url == $webform) ? ' selected="selected"' : ''; ?>>Split test: <?php echo $this_form->name; ?></option>
    <?php endforeach; ?>
</select>
<?php endforeach; ?>

<input type="hidden"
    name="<?php echo $this->widgetOptionsName; ?>[submit]"
    value="1"/>

<div style="margin-bottom: 13px;">
<a id="<?php echo $this->widgetOptionsName; ?>-form-preview" class="<?php echo $this->widgetOptionsName; ?>-form-preview" href="#" target="_blank">Preview form</a>
</div>
<?php else: ?>
This AWeber account does not currently have any completed Sign Up Forms.
<div style="margin-top: 13px; margin-bottom: 13px;">
    Please <a href="https://www.aweber.com/users/web_forms/index">create a web
    form</a> in order to place it on your Wordpress blog.
</div>
<?php endif; ?>
<script type="text/javascript">
    jQuery(document).ready(function() {
        function hideFormSelectors() {
            jQuery('.<?php echo $this->widgetOptionsName; ?>-form-select').each(function() {
                jQuery(this).hide();
            });
        }

        function listDropDown() {
            return jQuery('.<?php echo $this->widgetOptionsName; ?>-list');
        }

        function currentFormDropDown() {
            var list;
            listDropDown().each(function() {
                list = jQuery(this).val();
            });
            if (list != "") {
                return jQuery('.<?php echo $this->widgetOptionsName; ?>-' + list + '-webform');
            }
            return undefined;
        }

        function updateViewableFormSelector() {
            hideFormSelectors();
            var dropdown = currentFormDropDown();
            if (dropdown != undefined) {
                dropdown.each(function() {
                    jQuery(this).show();
                });
            }
        }

        function updatePreviewLink() {
            var form_url = "";
            var preview = jQuery('.<?php echo $this->widgetOptionsName; ?>-form-preview');
            var form_dropdown = currentFormDropDown();
            if (form_dropdown != undefined) {
                form_dropdown.each(function() {
                    form_url = jQuery(this).val();
                });
            }
            if (form_url == "") {
                preview.each(function() {
                    jQuery(this).attr('href', '#');
                    jQuery(this).hide();
                });
            } else {
                form_url = form_url.split('/');
                var form_id = form_url.pop();
                var form_type = form_url.pop();
                if (form_type == 'web_form_split_tests') {
                    preview.each(function() {
                        jQuery(this).attr('href', '#');
                        jQuery(this).hide();
                    });
                } else {
                    preview.each(function() { jQuery(this).show(); });
                    var hash = form_id % 100;
                    hash = ((hash < 10) ? '0' : '') + hash;
                    preview.each(function() {
                        jQuery(this).attr('href', 'https://forms.aweber.com/form/' + hash + '/' + form_id + '.html');
                    });
                }
            }
        }

        updateViewableFormSelector();
        updatePreviewLink();

        jQuery(document.body).on('change', '.<?php echo $this->widgetOptionsName; ?>-list', function() {
            updateViewableFormSelector();
            var form_dropdown = currentFormDropDown();
            if (form_dropdown !== undefined) {
                form_dropdown.val('');
            }
            updatePreviewLink();
        });
        jQuery('.<?php echo $this->widgetOptionsName; ?>-form-select').each(function() {
            jQuery(this).change(function() {
                updatePreviewLink();
            });
        });
    });
</script>

<?php
        $this->_end_response();
    }

    /**
        * Get web form snippet
        *
        * Retrieve webform snippet to be inserted in blog page.
        * @return string
        */
    function getWebformSnippet() {
        $options = get_option($this->widgetOptionsName);
        return $options['form_snippet'];
    }
}
?>
